//
//  CollectionTypes.swift
//  Optional-1249
//
//  Created by Frank Jüstel on 19.12.15.
//  Copyright © 2015 JSM Arts. All rights reserved.
//

import Foundation

enum KundenGruppe {
	case Interessent
	case Kunde
	case AltKunde
}

class Kunde: CustomStringConvertible {
	var name: String
	var gruppe: KundenGruppe

	var description: String {
		return "Kunde(\(name), \(gruppe))"
	}
	
	init(name:String, gruppe: KundenGruppe) {
		self.name = name
		self.gruppe = gruppe
	}
}

func collectionTypes() {
	array()
	sets()
	dictionaries()
	
	let zahlen = [1, 2, 3, 4, 5, 6, 7, 8, 9, 99, 12, 88, 11]
	
	var treffer = [Int]()
	for zahl in zahlen {
		if zahl > 7 {
			treffer.append(zahl)
		}
	}
	print(treffer)

	print(zahlen.filter { $0 > 7 }.map { $0 * 2 }.reduce(0) { $0 + $1 } )
	
	let kunden = [
		Kunde(name: "Willi", gruppe: .Interessent),
		Kunde(name: "Franky", gruppe: .Kunde),
		Kunde(name: "Walter", gruppe: .Kunde),
		Kunde(name: "Peter", gruppe: .AltKunde),
	]
	print(kunden.dynamicType)

	let mappedKunden = kunden.filter { $0.gruppe == .Kunde }.map { Kunde(name: $0.name, gruppe: .AltKunde) }
	print(mappedKunden)
}


func dictionaries() {
	print(__FUNCTION__)
	
	//var numbers = [Int : String]()
	var numbers = [0 : "Null", 1 : "Eins", 2 : "Zwei"]
	print(numbers.dynamicType)
	print(numbers[0])
	print(numbers[2])
	print(numbers[7])
	
	numbers[7] = "Sieben"
	numbers.updateValue("seven", forKey: 7)
	print(numbers[7])
	
	//numbers.removeValueForKey(7)
	numbers[7] = nil
	print("wir haben \(numbers.count) Elemente")
	print(numbers)
	
	var users = ["a@b.com" : "willi", "frank.juestel@jsmarts.de" : "Franky"]
	print(users["frank.juestel@jsmarts.de"])
	
	if let user = users["a@b.com"] {
		print("name = \(user)")
	}
	
	for value in numbers.values {
		print("Wert: \(value)")
	}

	for key in numbers.keys {
		print("Wert: \(key) = \(numbers[key]!)")
	}
	
	for (key, value) in numbers {
		print("\(key) = \(value)")
	}
}

func sets() {
	print(__FUNCTION__)
	//var shopingList = Set<String>()
	var shopingList: Set = ["chocolate", "cheese", "butter", "cheese"]
	print(shopingList.dynamicType)

	shopingList.insert("milk")
	shopingList.insert("cheese")
	shopingList.insert("eggs")
	
	print(shopingList)
	
	shopingList.insert("cheese")
	shopingList.remove("butter")
	print(shopingList)
	
	for item in shopingList {
		print("item = \(item)")
	}
	
	let set1: Set = [1, 2, 3, 4, 5, 6]
	let set2: Set = [5, 6, 7, 8, 9]
	let set3: Set = [3, 4, 5]
	
	let intersection = set1.intersect(set2)
	print(intersection)
	
	let union = set1.union(set2).sort()
	print(union)
	
	let substraction = set1.subtract(set2).sort()
	print(substraction)
	
	let exclusive = set1.exclusiveOr(set2).sort()
	print(exclusive)
	
	print(set3.isSubsetOf(set1))
}

func array() {
	print(__FUNCTION__)
	
//	var someInts:Array<Int> = Array<Int>()
	var someInts = [Int]()
	print(someInts.dynamicType)

	for i in 1...10 {
		someInts.append(i * 2)
	}
	
	print(someInts)
	
	let threeDoubles = [Double](count: 3, repeatedValue: 0.0)
	let anotherThreeDoubles = [Double](count: 3, repeatedValue: 2.5)
	let sixDoubles = threeDoubles + anotherThreeDoubles
	print(sixDoubles)
	
	var shopingList = ["eggs", "milk"]
	print(shopingList.dynamicType)
	
	shopingList.append("toast")
	
	shopingList += ["baking powder"]
	shopingList += ["chocolate", "cheese", "butter", "cheese"]

	print(shopingList.isEmpty)
	print("in der shopongList sind \(shopingList.count) items")
	print(shopingList)
	
	print(shopingList.first)
	print(shopingList.last)
	print(shopingList[0])
	
	print(shopingList[2...3])
	shopingList[2...3] = ["wine", "1", "3", "9"]
	print(shopingList)
	
	shopingList.insert("apples", atIndex: 0)
	print(shopingList)
	
	shopingList.removeAtIndex(0)
	print(shopingList.indexOf("cheese"))
	
	for index in 0 ..< shopingList.count {
		let item = shopingList[index]
		print("Position \(index) = \(item)")
	}
	
	for item in shopingList {
		print("item = \(item)")
	}
	
	for (index, item) in shopingList.enumerate() {
		print("Position \(index) = \(item)")
	}
	
	
}