//
//  StructuresAndClasses.swift
//  Optional-1249
//
//  Created by Frank Jüstel on 20.12.15.
//  Copyright © 2015 JSM Arts. All rights reserved.
//

import Foundation

class Person {
	var name: String
	weak var kreditkarte: Kreditkarte?
	
	init(name: String) {
		self.name = name
	}

	func addKreditkarte() {
		kreditkarte = Kreditkarte(person: self)
	}
	
	deinit {
		print("Die Person ist weg....")
	}
}

class Kreditkarte {
	var person: Person
	
	init(person: Person) {
		self.person = person
	}
	
	deinit {
		print("Die Kreditkarte ist weg....")
	}
}

struct Origin {
	var x = 0.0
	var y = 0.0
	
	mutating func moveByX(deltaX: Double, y deltaY:Double) {
		x += deltaX
		y += deltaY
	}
}

class Rechteck: CustomStringConvertible {
	var laenge: Double
	var breite: Double {
		willSet {
			if breite != newValue {
				brechungsindex = nil
			}
		}
	}
	
	var flaeche: Double {
		return laenge * breite
	}
	
	var description: String {
		return "Rechteck(laenge: \(laenge), breite: \(breite))"
	}
	
	lazy var brechungsindex: Double! = {
		print("brechungsindex bestimmen...")
		NSThread.sleepForTimeInterval(0.5)
		print("...fertig")
		return self.breite * 10
	}()
	
	init (laenge: Double = 1, breite: Double = 1) {
		self.laenge = laenge
		self.breite = breite
		lustig()
	}

	deinit {
		print("ich war ein Rechteck und muss jetzt auch weg....")
	}
	
	func lustig() {
		print("Lustig im Rechteck: \(laenge), \(breite) :-)")
	}
}

class Quader: Rechteck {
	var hoehe: Double
	
	override var flaeche: Double {
		return super.flaeche * 0.9
	}
	
	var volumen: Double {
		return flaeche * hoehe
	}
	
	override var description: String {
		return "Quader(laenge: \(laenge), breite: \(breite), hoehe: \(hoehe)) volumen = \(volumen)"
	}
	
	init?(laenge: Double = 1, breite: Double = 1, hoehe: Double = 1) {
		self.hoehe = hoehe
		super.init(laenge: laenge, breite: breite)
		
		if volumen >= 500 { return nil }
	}
	
	deinit {
		print("ich war ein Quader und muss jetzt leider weg....")
	}
	
	override func lustig() {
		super.lustig()
		print("Lustig im Quader: \(laenge), \(breite), \(hoehe) :-)")
	}
}

func structuresAndClasses() {
	//let rechteck = Rechteck(breite: 17, laenge: 5)
	//print(rechteck)
	
	if let quader = Quader(laenge: 5, breite: 5, hoehe: 10) {
		print(quader)
		
		print("1 \(quader.brechungsindex)")

		quader.breite = 5
		print("2 \(quader.brechungsindex)")

		quader.breite = 6
		print("3 \(quader.brechungsindex)")
	} else {
		print("tut mir leid, das Ding ist einfach zu groß")
	}
	
	var origin = Origin(x: 10, y: 10)
	print(origin)
	
	origin.moveByX(5, y: 7)
	print(origin)
	
	let personWilli = Person(name: "Willi")
	print(personWilli.name)
	
	personWilli.addKreditkarte()
}